from rest_framework import serializers
from event.models import Event, ActivityTag, Place, ActivityTemplate


class EventSerializer(serializers.HyperlinkedModelSerializer):
    created_by = serializers.ReadOnlyField(source='created_by.username')

    class Meta:
        model = Event
        fields = ('url', 'id', 'title', 'slug', 'created_by', 'creation_date',
                  'description', 'beginning_date', 'ending_date')


# TODO rajouter des permissions
class PlaceSerializer(serializers.HyperlinkedModelSerializer):
    event = EventSerializer(allow_null=True)

    class Meta:
        model = Place
        fields = ('url', 'id', 'name', 'description', 'event')

    def update(self, instance, validated_data):
        try:
            data = validated_data.pop('event')
            event = Event.objects.get_object_or_404(**data)
        except KeyError:
            event = None
        [setattr(instance, key, value)
         for key, value in validated_data.items()]
        setattr(instance, 'event', event)
        instance.save()
        return instance

    def create(self, validated_data):
        ModelClass = self.Meta.model
        try:
            data = validated_data.pop('event')
            event = Event.objects.get_object_or_404(**data)
        except KeyError:
            event = None
        instance = ModelClass.objects.create(event=event, **validated_data)
        return instance


# TODO rajouter des permissions
class ActivityTagSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        model = ActivityTag
        fields = ('url', 'id', 'name', 'is_public', 'color',)


# TODO rajouter des permissions
class ActivityTemplateSerializer(serializers.HyperlinkedModelSerializer):
    event = serializers.ReadOnlyField(source='event.title')
    tags = ActivityTagSerializer(many=True)

    class Meta:
        model = ActivityTemplate
        fields = ('url', 'id', 'title', 'event', 'is_public', 'has_perm',
                  'min_perm', 'max_perm', 'description', 'remarks', 'tags',)

    def update(self, instance, validated_data):
        tags_data = validated_data.pop('tags')
        [setattr(instance, key, value)
         for key, value in validated_data.items()]
        instance.save()
        tags = [ActivityTag.objects.get_or_create(**tag_data)[0]
                for tag_data in tags_data]
        instance.tags = tags
        return instance

    def create(self, validated_data):
        tags_data = validated_data.pop('tags')
        activity_template = ActivityTemplate.objects.create(**validated_data)
        tags = [ActivityTag.objects.get_or_create(**tag_data)[0]
                for tag_data in tags_data]
        activity_template.tags = tags
        return activity_template
